/*
 *  Created on 26.11.2003
 *
 */
package seco.semcocoon.transformation;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.ModelFactory;

import java.io.InputStream;
import java.io.Serializable;
import java.io.StringReader;

import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.cocoon.caching.CacheableProcessingComponent;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceResolver;
import org.apache.excalibur.source.SourceValidity;
import org.xml.sax.SAXException;

import vbsearch.VBSConf;
import vbsearch.VBSearch;
import vbsearch.impl.jena.JenaVBSQuery;

/**
 * @author     jiemakel
 * @created    10. joulukuuta 2003
 */
public class OgtTransformer extends AbstractSerializedXMLTransformer implements Configurable, CacheableProcessingComponent {
    private final static OntModelSpec ONT_MODEL_SPEC  = OntModelSpec.OWL_LITE_MEM_RDFS_INF;
    private static String BASE_NS                     = "http://www.cs.helsinki.fi/group/seco/ontologies/ontogator/vbsearch#";
    static Source initializationSource                = null;
    static boolean initialized                        = false;
    static String key                                 = null;
    static String[] literalBookmarkIndexProperties;
    static String[] literalCategoryIndexProperties;

    static VBSearch ontogator                         = null;
    static Configuration[] sources;
    static SourceValidity validity                    = null;


    /**
     *  Description of the Method
     *
     * @param  conf                        Description of the Parameter
     * @exception  ConfigurationException  Description of the Exception
     */
    public void configure(Configuration conf)
         throws ConfigurationException {
        sources = conf.getChildren("source");
        StringBuffer tmp      = new StringBuffer();
        for (int i = 0; i < sources.length; i++) {
            tmp.append(sources[i].getValue());
        }
        Configuration[] temp  = conf.getChildren("literalBookmarkIndexProperty");
        if (temp != null && temp.length > 0) {
            VBSConf.LITERAL_BOOKMARK_INDEX_PROPERTIES = new String[temp.length];
            for (int i = 0; i < temp.length; i++) {
                VBSConf.LITERAL_BOOKMARK_INDEX_PROPERTIES[i] = temp[i].getValue();
                tmp.append(temp[i].getValue());
            }
        }
        temp = conf.getChildren("literalCategoryIndexProperty");
        if (temp != null && temp.length > 0) {
            VBSConf.LITERAL_CATEGORY_INDEX_PROPERTIES = new String[temp.length];
            for (int i = 0; i < temp.length; i++) {
                VBSConf.LITERAL_CATEGORY_INDEX_PROPERTIES[i] = temp[i].getValue();
                tmp.append(temp[i].getValue());
            }
        }
        key = tmp.toString();
    }


    /**
     *  Description of the Method
     *
     * @param  queryStr          Description of the Parameter
     * @exception  SAXException  Description of the Exception
     */
    public void generate(String queryStr) throws SAXException {
        try {
            initOntogator(this.resolver);
            OntModel query  = ModelFactory.createOntologyModel(VBSConf.ONT_MODEL_SPEC, null);
            ontogator.executeQuery(new JenaVBSQuery((OntModel)query.read(new StringReader(queryStr), VBSConf.BASE_URI)),
                contentHandler);
        } catch (Exception e) {
            throw new SAXException(e);
        }
    }


    /**
     * @return
     */
    public Serializable getKey() {
        if (!isInitialized()) {
            return null;
        }
        else {
            return key;
        }
    }


    /**
     * @return
     */
    public SourceValidity getValidity() {
        if (!isInitialized()) {
            return null;
        }
        else {
            return validity;
        }
    }


    /**
     *  Description of the Method
     *
     * @param  resolver
     */
    public static synchronized void initOntogator(SourceResolver resolver) {
        if (ontogator == null
             || (validity.isValid() != 1
             && (validity.isValid() == -1 || validity.isValid(initializationSource.getValidity()) == -1))) {
            initialized = false;
            ontogator = new VBSearch();
            OntModel inputOnt  = ModelFactory.createOntologyModel(ONT_MODEL_SPEC, null);
            for (int i = 0; i < sources.length; i++) {
                try {
                    Source source         = resolver.resolveURI(sources[i].getValue());
                    System.out.println("Trying to initialize Ontogator from " + source.getURI());
                    InputStream response  = source.getInputStream();
                    inputOnt.read(response, BASE_NS);
                    ontogator.add(inputOnt, "defaultModel");
                    System.out.println("Ontogator initialized from " + source.getURI());
					System.out.println(ontogator.getIndex().toString());
                    initialized = true;
                    initializationSource = source;
                    validity = initializationSource.getValidity();
                    return;
                } catch (Exception e) {
                    try {
                        System.out.println("Initializing from source " + sources[i].getValue() + " caused an exception: " + e.getMessage());
                    } catch (ConfigurationException e2) {}
                }
            }
            initialized = true;
        }
    }


    /**
     * @return
     */
    private static synchronized boolean isInitialized() {
        return initialized;
    }
}

